-- SECTION 1 ---- CONFIGURE THE DISTRIBUTOR SERVERS

-- Step1 - Configure the Distribution DB nodes (AG Replicas) to act as a distributor
:CONNECT SQL2K16-AG01
EXECUTE sp_adddistributor @distributor = @@ServerName, @password = 'Pass@word1'
GO 
:CONNECT SQL2K16-AG02
EXECUTE sp_adddistributor @distributor = @@ServerName, @password = 'Pass@word1'
GO
:CONNECT SQL2K16-AG03
EXECUTE sp_adddistributor @distributor = @@ServerName, @password = 'Pass@word1'
GO

-- Step2 - Configure the Distribution Database
:Connect SQL2K16-AG01
USE master
EXECUTE sp_adddistributiondb @database = 'DistributionDB', @security_mode = 1;
GO
ALTER DATABASE [DistributionDB] SET RECOVERY FULL;
GO
BACKUP DATABASE [DistributionDB] TO DISK = N'NUL';
GO

-- Step 3 - Create AG for the Distribution DB.
:Connect SQL2K16-AG01
-- Create the Availability Group
CREATE AVAILABILITY GROUP [DistributionDB_AG]
FOR DATABASE [DistributionDB]
REPLICA ON 'SQL2K16-AG01'
WITH (ENDPOINT_URL = N'TCP://SQL2K16-AG01.SQLSKILLSDEMOS.com:5022', 
		 FAILOVER_MODE = AUTOMATIC, 
		 AVAILABILITY_MODE = SYNCHRONOUS_COMMIT, 
		 BACKUP_PRIORITY = 50, 
		 SECONDARY_ROLE(ALLOW_CONNECTIONS = ALL), 
		 SEEDING_MODE = AUTOMATIC),
N'SQL2K16-AG02' WITH (ENDPOINT_URL = N'TCP://SQL2K16-AG02.SQLSKILLSDEMOS.com:5022', 
	 FAILOVER_MODE = AUTOMATIC, 
	 AVAILABILITY_MODE = SYNCHRONOUS_COMMIT, 
	 BACKUP_PRIORITY = 50, 
	 SECONDARY_ROLE(ALLOW_CONNECTIONS = ALL), 
	 SEEDING_MODE = AUTOMATIC),
N'SQL2K16-AG03' WITH (ENDPOINT_URL = N'TCP://SQL2K16-AG03.SQLSKILLSDEMOS.com:5022', 
	 FAILOVER_MODE = AUTOMATIC, 
	 AVAILABILITY_MODE = SYNCHRONOUS_COMMIT, 
	 BACKUP_PRIORITY = 50, 
	 SECONDARY_ROLE(ALLOW_CONNECTIONS = ALL), 
	 SEEDING_MODE = AUTOMATIC);
 GO


:Connect SQL2K16-AG02
ALTER AVAILABILITY GROUP [DistributionDB_AG] JOIN
GO  
ALTER AVAILABILITY GROUP [DistributionDB_AG] GRANT CREATE ANY DATABASE
GO


:Connect SQL2K16-AG03
ALTER AVAILABILITY GROUP [DistributionDB_AG] JOIN
GO  
ALTER AVAILABILITY GROUP [DistributionDB_AG] GRANT CREATE ANY DATABASE
GO

--STEP4 - Create the Listener for the Availability Group. This is very important.
:Connect SQL2K16-AG01

USE [master]
GO
ALTER AVAILABILITY GROUP [DistributionDB_AG]
ADD LISTENER N'DistributionDBList' (
WITH IP
((N'192.168.10.37', N'255.255.255.0')) , PORT=1500);
GO

-- STEP 5 - Enable SQL2K16-AG02 also as a Distributor
:CONNECT SQL2K16-AG02
EXEC sp_adddistributiondb @database = 'DistributionDB', @security_mode = 1;
GO
-- Enable SQL2K16-AG0 also as a Distributor
:CONNECT SQL2K16-AG03
EXEC sp_adddistributiondb @database = 'DistributionDB', @security_mode = 1;
GO



--STEP 6 - On all Distributor Nodes Configure the Publisher Details 
:CONNECT SQL2K16-AG01
EXEC sp_addDistPublisher @publisher = 'SQL2K16-AG01', @distribution_db = 'DistributionDB', 
	@working_directory = '\\SQL2K16-AG01\Repl\'
GO
:CONNECT SQL2K16-AG02
EXEC sp_addDistPublisher @publisher = 'SQL2K16-AG01', @distribution_db = 'DistributionDB', 
	@working_directory = '\\SQL2K16-AG01\Repl\'
GO


-- SECTION 2 ---- CONFIGURE THE PUBLISHER SERVER
:CONNECT SQL2K16-AG01
EXEC sp_addDistributor @distributor = 'DistributionDBList', -- Listener for the Distribution DB.	
	@password = 'Pass@word1'
GO

-- SECTION 3 ---- CONFIGURE THE SUBSCRIBERS 
-- On Publisher, create the publication as one would normally do.
-- On the Secondary replicas of the Distribution DB, add the Subscriber as a linked server.
:CONNECT SQL2K16-DAG01
EXEC master.dbo.sp_addlinkedserver @server = N'SQLNODE5', @srvproduct=N'SQL Server'
 /* For security reasons the linked server remote logins password is changed with ######## */
EXEC master.dbo.sp_addlinkedsrvlogin @rmtsrvname=N'SQLNODE5',@useself=N'True',@locallogin=NULL,@rmtuser=NULL,@rmtpassword=NULL